SystemOrganization addCategory: #'PetitRegex-Core'!
SystemOrganization addCategory: #'PetitRegex-Tests'!

PPCompositeParser subclass: #PPRegexParser
	instanceVariableNames: 'regex atom anyAtom beginOfLineAtom endOfLineAtom characterAtom rangePieceSpec number groupAtom escapeAtom characterGroup quantifier'
	classVariableNames: 'EscapeMap'
	poolDictionaries: ''
	category: 'PetitRegex-Core'!

!PPRegexParser class methodsFor: 'initialization' stamp: 'lr 2/8/2010 08:40'!
condition: aBlock
	^ [ :stream | (aBlock value: stream) ifFalse: [ PPFailure new ] ] asParser! !

!PPRegexParser class methodsFor: 'initialization' stamp: 'lr 2/8/2010 10:13'!
initialize
	EscapeMap := Dictionary new.
	EscapeMap
		"only at beginning of string"
		at: $A put: (self condition: [ :stream | stream position = 0 ]);
		"only at end of string"		
		at: $Z put: (self condition: [ :stream | stream atEnd ]);
		"newline"
		at: $n put: Character lf asParser;
		"carriage return"
		at: $r put: Character cr asParser;
		"tab"
		at: $t put: Character tab asParser;
		"digit"
		at: $d put: #digit asParser;
		"not digit"
		at: $D put: #digit asParser negate;
		"word"
		at: $w put: #word asParser;
		"not word"
		at: $W put: #word asParser negate;
		"whitespace"
		at: $s put: #space asParser;
		"not whitespace"
		at: $S put: #space asParser negate! !

!PPRegexParser methodsFor: 'grammar-atoms' stamp: 'lr 2/7/2010 23:28'!
anyAtom
	^ $. asParser map: [ :char | #any asParser ]! !

!PPRegexParser methodsFor: 'grammar' stamp: 'lr 2/8/2010 00:18'!
atom
	^ escapeAtom / anyAtom / beginOfLineAtom / endOfLineAtom / characterGroup / groupAtom / characterAtom! !

!PPRegexParser methodsFor: 'grammar-atoms' stamp: 'lr 2/8/2010 00:40'!
beginOfLineAtom
	^ $^ asParser map: [ :char | [ :stream | 
		(stream position = 0 or: [ (stream skip: -1; next) = Character cr ])
			ifFalse: [ PPFailure at: stream position ] ] asParser ]! !

!PPRegexParser methodsFor: 'grammar-atoms' stamp: 'lr 2/8/2010 00:16'!
characterAtom
	^ $) asParser negate map: [ :char | char asParser ]! !

!PPRegexParser methodsFor: 'grammar-atoms' stamp: 'lr 2/8/2010 00:29'!
characterGroup
	^ $[ asParser , $^ asParser optional , [ :stream |
			| set |
			set := OrderedCollection new.
			[ stream atEnd or: [ stream peek = $] ] ] whileFalse: [
				(stream peek = $- and: [ set notEmpty ])
					ifTrue: [ set addAll: (set removeLast to: (stream next; next)) ]
					ifFalse: [ set add: stream next ] ].
			set ] asParser , $] asParser map: [ :open :negate :set :close |
				negate isNil
					ifTrue: [ (PPPredicateParser anyOf: set asSet)  ]
					ifFalse: [ (PPPredicateParser anyOf: set asSet) negate ] ]! !

!PPRegexParser methodsFor: 'grammar-atoms' stamp: 'lr 2/8/2010 00:40'!
endOfLineAtom
	^ $$ asParser map: [ :char | [ :stream | 
		(stream atEnd or: [ stream peek = Character cr ])
			ifFalse: [ PPFailure at: stream position ]  ] asParser ]! !

!PPRegexParser methodsFor: 'grammar-atoms' stamp: 'lr 2/8/2010 08:43'!
escapeAtom
	^ $\ asParser , #any asParser map: [ :escape :char | EscapeMap at: char ifAbsent: [ char asParser ] ]! !

!PPRegexParser methodsFor: 'grammar-atoms' stamp: 'lr 2/8/2010 00:11'!
groupAtom
	^ $( asParser , regex , $) asParser ==> #second! !

!PPRegexParser methodsFor: 'other' stamp: 'lr 2/8/2010 19:25'!
number
	^ #digit asParser plus flatten ==> [ :token | token asInteger ]! !

!PPRegexParser methodsFor: 'grammar-operators' stamp: 'lr 3/1/2010 16:33'!
oneOrMorePiece
	"+	1 or more times"
	
	^ $+ asParser , quantifier map: [ :op :qu | qu on: nil min: 1 max: SmallInteger maxVal ]! !

!PPRegexParser methodsFor: 'grammar-operators' stamp: 'lr 3/1/2010 16:24'!
quantifier
	^ ($+ asParser ==> [ :node | PPRepeatingParser ])
	/ ($? asParser ==> [ :node | PPLazyQuantifierStub ])
	/ (nil asParser ==> [ :node | PPGreedyQuantifierStub ])! !

!PPRegexParser methodsFor: 'grammar-operators' stamp: 'lr 3/1/2010 16:33'!
rangePiece
	^ ${ asParser , rangePieceSpec ,  $} asParser , quantifier 
		map: [ :op :ra :cl :qu | qu on: nil min: ra first max: ra last ]! !

!PPRegexParser methodsFor: 'grammar-operators' stamp: 'lr 3/1/2010 16:31'!
rangePieceSpec
	"{n,m}	at least n but not more than m times
	{n,}	at least n times
	{n}	exactly n times"
	
	^ number optional , $, asParser optional , number optional
		map: [ :n :op :m | (n ifNil: [ 0 ]) to: (m ifNil: [ SmallInteger maxVal ]) ]! !

!PPRegexParser methodsFor: 'grammar' stamp: 'lr 3/1/2010 16:32'!
regex
	^ PPExpressionParser new
		term: atom;
		group: [ :g |.
			g postfix: self zeroOrOnePiece do: [ :at :op | op setParser: at ].
			g postfix: self zeroOrMorePiece do: [ :at :op | op setParser: at ].
			g postfix: self oneOrMorePiece do: [ :at :op | op setParser: at ].
			g postfix: self rangePiece do: [ :at :op | op setParser: at ] ];
		group: [ :g |
			g left: nil do: [ :left :op :right | left , right ] ];
		group: [ :g |
			g left: $| do: [ :left :op :right | left / right ] ];
		yourself! !

!PPRegexParser methodsFor: 'private' stamp: 'lr 3/1/2010 16:40'!
resolveStubs: aParser
	"In this post-processing step we need to fix the quantifiers."
	
	aParser followSets keysAndValuesDo: [ :each :set |
		(each class = PPGreedyQuantifierStub or: [ each class = PPLazyQuantifierStub ]) ifTrue: [
			each def: (each resolve: (set size = 1
				ifTrue: [ set anyOne ]
				ifFalse: [ PPChoiceParser withAll: set ])) ] ].
	^ aParser! !

!PPRegexParser methodsFor: 'accessing' stamp: 'lr 3/1/2010 16:17'!
start
	^ regex end ==> [ :node | self resolveStubs: node ]! !

!PPRegexParser methodsFor: 'grammar-operators' stamp: 'lr 3/1/2010 16:33'!
zeroOrMorePiece
	"*	0 or more times"
		
	^ $* asParser ,  quantifier map: [ :op :qu | qu on: nil min: 0 max: SmallInteger maxVal ]! !

!PPRegexParser methodsFor: 'grammar-operators' stamp: 'lr 3/1/2010 16:33'!
zeroOrOnePiece
	"?	0 or 1 times"

	^ $? asParser ,  quantifier map: [ :op :qu | qu on: nil min: 0 max: 1 ]! !

PPRepeatingParser subclass: #PPGreedyQuantifierStub
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitRegex-Core'!

!PPGreedyQuantifierStub methodsFor: 'protected' stamp: 'lr 3/3/2010 15:57'!
resolve: aParser
	^ (min = 0 and: [ max = SmallInteger maxVal ])
		ifTrue: [ parser starGreedy: aParser ]
		ifFalse: [
			(min = 1 and: [ max = SmallInteger maxVal ])
				ifTrue: [ parser plusGreedy: aParser ]
				ifFalse: [ 
					(min = 0 and: [ max = 1])
						ifTrue: [ parser optional ]
						ifFalse: [ self error: 'Invalid quantifier range' ] ] ]! !

PPRepeatingParser subclass: #PPLazyQuantifierStub
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitRegex-Core'!

!PPLazyQuantifierStub methodsFor: 'protected' stamp: 'lr 3/3/2010 15:57'!
resolve: aParser
	^ (min = 0 and: [ max = SmallInteger maxVal ])
		ifTrue: [ parser starLazy: aParser ]
		ifFalse: [ 
			(min = 1 and: [ max = SmallInteger maxVal ])
				ifTrue: [ parser plusLazy: aParser ]
				ifFalse: [ 
					(min = 0 and: [ max = 1])
						ifTrue: [ parser optional ]
						ifFalse: [ self error: 'Invalid quantifier range' ] ] ]! !

TestCase subclass: #PPRegexTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitRegex-Tests'!

!PPRegexTest class methodsFor: 'accessing' stamp: 'lr 2/8/2010 10:14'!
packageNamesUnderTest
	^ #('PetitRegex')! !

!PPRegexTest methodsFor: 'utilties' stamp: 'lr 2/8/2010 00:49'!
compileRegex: aString
	"Compile the regex and answer the matcher, or answer nil if compilation fails."

	| syntaxTree |
	syntaxTree := self parserClass parse: aString.
	^ syntaxTree isPetitFailure ifFalse: [ syntaxTree ]! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
henryReadme
	self error: 'The tests in this category are based on the ones in Henry Spencer''s regexp.c package.'! !

!PPRegexTest methodsFor: 'accessing' stamp: 'lr 2/8/2010 00:46'!
parserClass
	^ PPRegexParser! !

!PPRegexTest methodsFor: 'utilties' stamp: 'lr 3/1/2010 21:33'!
runMatcher: aParser with: aString expect: aBoolean withSubexpressions: anArray
	| copy got |
	"copy := aParser
		copy: aString
		translatingMatchesUsing: [ :each | each ].
	self 
		assert: copy = aString
		description: 'Copying: expected ' , aString printString , ', but got ' , copy printString."
	got := aParser matchesIn: aString.
	self
		assert: got notEmpty = aBoolean 
		description: 'Searching: expected ' , aBoolean printString , ', but got ' , got printString.
	"(anArray isNil or: [ aParser supportsSubexpressions not ])
		ifTrue: [ ^ self ].
	1 to: anArray size by: 2 do: [ :index |
		| sub subExpect subGot |
		sub := anArray at: index.
		subExpect := anArray at: index + 1.
		subGot := aParser subexpression: sub.
		self
			assert: subExpect = subGot
			description: 'Subexpression ' , sub printString , ': expected ' , subExpect printString , ', but got ' , subGot printString ]"! !

!PPRegexTest methodsFor: 'utilties' stamp: 'lr 2/8/2010 00:52'!
runRegex: anArray
	"Run a clause anArray against a set of tests. Each clause is an array with a regex source string followed by sequence of 3-tuples. Each three-element group is one test to try against the regex, and includes: 1) test string; 2) expected result; 3) expected subexpression as an array of (index, substring), or nil."

	| source matcher |
	source := anArray first.
	matcher := self compileRegex: source.
	matcher isNil
		ifTrue: [
			(anArray at: 2) isNil
				ifFalse: [ self signalFailure: 'Compilation failed, should have succeeded: ' , source printString ] ]
		ifFalse: [
			(anArray at: 2) isNil
				ifTrue: [ self signalFailure: 'Compilation succeeded, should have failed: ' , source printString ]
				ifFalse: [
					2 to: anArray size by: 3 do: [ :index | 
						self 
							runMatcher: matcher
							with: (anArray at: index)
							expect: (anArray at: index + 1)
							withSubexpressions: (anArray at: index + 2) ] ] ]! !

!PPRegexTest methodsFor: 'testing-protocol' stamp: 'lr 2/8/2010 00:45'!
testCaseInsensitive
	| matcher |
	matcher := self matcherClass forString: 'the quick brown fox' ignoreCase: true.
	self assert: (matcher search: 'the quick brown fox').
	self assert: (matcher search: 'The quick brown FOX').
	self assert: (matcher search: 'What do you know about the quick brown fox?').
	self assert: (matcher search: 'What do you know about THE QUICK BROWN FOX?')! !

!PPRegexTest methodsFor: 'testing-protocol' stamp: 'lr 2/8/2010 00:45'!
testCaseSensitive
	| matcher |
	matcher := self matcherClass forString: 'the quick brown fox' ignoreCase: false.
	self assert: (matcher search: 'the quick brown fox').
	self deny: (matcher search: 'The quick brown FOX').
	self assert: (matcher search: 'What do you know about the quick brown fox?').
	self deny: (matcher search: 'What do you know about THE QUICK BROWN FOX?')! !

!PPRegexTest methodsFor: 'testing-protocol' stamp: 'lr 2/8/2010 00:45'!
testCopyReplacingMatches
	"See that the match context is preserved while copying stuff between matches:"
	
	| matcher |
	matcher := self matcherClass forString: '\<\d\D+'.
	self assert: (matcher copy: '9aaa1bbb 8ccc' replacingMatchesWith: 'foo')
		= 'foo1bbb foo'! !

!PPRegexTest methodsFor: 'testing-protocol' stamp: 'lr 2/8/2010 00:45'!
testCopyTranslatingMatches
	| matcher |
	matcher := self matcherClass forString: '\w+'.
	self assert: (matcher copy: 'now is  the   time    ' translatingMatchesUsing: [ :each | each reverse ])
		= 'won si  eht   emit    '! !

!PPRegexTest methodsFor: 'testing-empty' stamp: 'lr 3/3/2010 15:52'!
testEmptyStringAtBeginningOfLine
	| matcher |
	matcher := self compileRegex: '^'.
	self
		assert: (matcher copy: 'foo1 bar1' , String cr , 'foo2 bar2' replacingMatchesWith: '*')
			= ('*foo1 bar1' , String cr , '*foo2 bar2')
		description: 'An empty string at the beginning of a line'! !

!PPRegexTest methodsFor: 'testing-empty' stamp: 'lr 3/3/2010 15:52'!
testEmptyStringAtBeginningOfWord
	| matcher |
	matcher := self compileRegex: '\<'.
	self
		assert: (matcher copy: 'foo bar' replacingMatchesWith: '*')
			= '*foo *bar'
		description: 'An empty string at the beginning of a word'! !

!PPRegexTest methodsFor: 'testing-empty' stamp: 'lr 3/3/2010 15:52'!
testEmptyStringAtEndOfLine
	| matcher |
	matcher := self compileRegex: '$'.
	self
		assert: (matcher copy: 'foo1 bar1' , String cr , 'foo2 bar2' replacingMatchesWith: '*')
			= ('foo1 bar1*', String cr , 'foo2 bar2*')
		description: 'An empty string at the end of a line'! !

!PPRegexTest methodsFor: 'testing-empty' stamp: 'lr 3/3/2010 15:53'!
testEmptyStringAtEndOfWord
	| matcher |
	matcher := self compileRegex: '\>'.
	self
		assert: (matcher copy: 'foo bar' replacingMatchesWith: '*')
			= 'foo* bar*'
		description: 'An empty string at the end of a word'! !

!PPRegexTest methodsFor: 'testing-empty' stamp: 'lr 3/3/2010 15:53'!
testEmptyStringAtWordBoundary
	| matcher |
	matcher := self compileRegex: '\b'.
	self
		assert: (matcher copy: 'foo bar' replacingMatchesWith: '*')
			= '*foo* *bar*'
		description: 'An empty string at a word boundary'! !

!PPRegexTest methodsFor: 'testing-empty' stamp: 'lr 3/3/2010 15:53'!
testEmptyStringNotAtWordBoundary
	| matcher |
	matcher := self compileRegex: '\B'.
	self
		assert: (matcher copy: 'foo bar' replacingMatchesWith: '*')
			= 'f*o*o b*a*r'
		description: 'An empty string not at a word boundary'! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry001
	self runRegex: #('abc'
		'abc' true (1 'abc')
		'xbc' false nil
		'axc' false nil
		'abx' false nil
		'xabcy' true (1 'abc')
		'ababc' true (1 'abc'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry002
	self runRegex: #('ab*c'
		'abc' true (1 'abc'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry003
	self runRegex: #('ab*bc'
		'abc' true (1 'abc')
		'abbc' true (1 'abbc')
		'abbbbc' true (1 'abbbbc'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry004
	self runRegex: #('ab+bc'	
		'abbc' true (1 'abbc')
		'abc' false nil
		'abq' false nil
		'abbbbc' true (1 'abbbbc'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry005
	self runRegex: #('ab?bc'
		'abbc' true (1 'abbc')
		'abc' true (1 'abc')
		'abbbbc' false nil
		'abc' true (1 'abc'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry006
	self runRegex: #('^abc$'
		'abc' true (1 'abc')
		'abcc' false nil
		'aabc' false nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry007
	self runRegex: #('^abc'
		'abcc' true (1 'abc'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry008
	self runRegex: #('abc$'
		'aabc' true (1 'abc'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry009
	self runRegex: #('^'
		'abc' true nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry010
	self runRegex: #('$'
		'abc' true nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry011
	self runRegex: #('a.c'
		'abc' true (1 'abc')
		'axc' true (1 'axc'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry012
	"Need to get creative to include the null character..."
	self runRegex: #('a.*c'	
		'axyzc' true (1 'axyzc')
		'axy zc' true (1 'axy zc') "testing that a dot matches a space"
		), (Array with: 'axy', (String with: 0 asCharacter), 'zc'), #(false nil "testing that a dot does not match a null"
		'axyzd' false nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry013
	self runRegex: #('.a.*'
		'1234abc' true (1 '4abc')
		'abcd' false nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry014
	self runRegex: #('a\w+c'
		' abbbbc ' true (1 'abbbbc')
		'abb bc' false nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry015
	self runRegex: #('\w+'
		'  	foobar	quux' true (1 'foobar')
		' 	~!!@#$%^&*()-+=\|/?.>,<' false nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry016
	self runRegex: #('a\W+c'
		'a   c' true (1 'a   c')
		'a bc' false nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry017
	self runRegex: #('\W+'
		'foo!!@#$bar' true (1 '!!@#$')
		'foobar' false nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry018
	self runRegex: #('a\s*c'
		'a   c' true (1 'a   c')
		'a bc' false nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry019
	self runRegex: #('\s+'
		'abc3457 sd' true (1 ' ')
		'1234$^*^&asdfb' false nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry020
	self runRegex: #('a\S*c'
		'aqwertyc' true (1 'aqwertyc')
		'ab c' false nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry021
	self runRegex: #('\S+'
		'     	asdf		' true (1 'asdf')
		' 	
			' false nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry022
	self runRegex: #('a\d+c'
		'a0123456789c' true (1 'a0123456789c')
		'a12b34c' false nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry023
	self runRegex: #('\d+'
		'foo@#$%123ASD #$$%^&' true (1 '123')
		'foo!!@#$asdfl;' false nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry024
	self runRegex: #('a\D+c'
		'aqwertyc' true (1 'aqwertyc')
		'aqw6ertc' false nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry025
	self runRegex: #('\D+'
		'1234 abc 456' true (1 ' abc ')
		'1234567890' false nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry026
	self runRegex: #('(f|o)+\b'
		'foo' true (1 'foo')
		' foo ' true (1 'foo'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry027
	self runRegex: #('\ba\w+' "a word beginning with an A"
		'land ancient' true (1 'ancient')
		'antique vase' true (1 'antique')
		'goofy foobar' false nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry028
	self runRegex: #('(f|o)+\B'
		'quuxfoobar' true (1 'foo')
		'quuxfoo ' true (1 'fo'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry029
	self runRegex: #('\Ba\w+' "a word with an A in the middle, match at A and further"
		'land ancient' true (1 'and')
		'antique vase' true (1 'ase')
		'smalltalk shall overcome' true (1 'alltalk')
		'foonix is better' false nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry030
	self runRegex: #('fooa\>.*'
		'fooa ' true nil
		'fooa123' false nil
		'fooa bar' true nil
		'fooa' true nil
		'fooargh' false nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry031
	self runRegex: #('\>.+abc'
		' abcde fg' false nil
		'foo abcde' true (1 ' abc')
		'abcde' false nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry032
	self runRegex: #('\<foo.*'
		'foo' true nil
		'foobar' true nil
		'qfoobarq foonix' true (1 'foonix')
		' foo' true nil
		' 12foo' false nil
		'barfoo' false nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry033
	self runRegex: #('.+\<foo'
		'foo' false nil
		'ab foo' true (1 'ab foo')
		'abfoo' false nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry034
	self runRegex: #('a[bc]d'
		'abc' false nil
		'abd' true (1 'abd'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry035
	self runRegex: #('a[b-d]e'
		'abd' false nil
		'ace' true (1 'ace'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry036
	self runRegex: #('a[b-d]'
		'aac' true (1 'ac'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry037
	self runRegex: #('a[-b]'
		'a-' true (1 'a-'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry038
	self runRegex: #('a[b-]'
		'a-' true (1 'a-'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry039
	self runRegex: #('a[a-b-c]' nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry040
	self runRegex: #('[k]'
		'ab' false nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry041
	self runRegex: #('a[b-a]' nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry042
	self runRegex: #('a[]b' nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry043
	self runRegex: #('a[' nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry044
	self runRegex: #('a]' 
		'a]' true (1 'a]'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry045
	self runRegex: #('a[]]b'
		'a]b' true (1 'a]b'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry046
	self runRegex: #('a[^bc]d'
		'aed' true (1 'aed')
		'abd' false nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry047
	self runRegex: #('a[^-b]c'
		'adc' true (1 'adc')
		'a-c' false nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry048
	self runRegex: #('a[^]b]c'
		'a]c' false nil
		'adc' true (1 'adc'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry049
	self runRegex: #('[\de]+'
		'01234' true (1 '01234')
		'0123e456' true (1 '0123e456')
		'0123e45g78' true (1 '0123e45'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry050
	self runRegex: #('[e\d]+' "reversal of the above, should be the same"
		'01234' true (1 '01234')
		'0123e456' true (1 '0123e456')
		'0123e45g78' true (1 '0123e45'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry051
	self runRegex: #('[\D]+'
		'123abc45def78' true (1 'abc'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry052
	self runRegex: #('[[:digit:]e]+'
		'01234' true (1 '01234')
		'0123e456' true (1 '0123e456')
		'0123e45g78' true (1 '0123e45'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry053
	self runRegex: #('[\s]+'
		'2  spaces' true (1 '  '))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry054
	self runRegex: #('[\S]+'
		'  word12!!@#$  ' true (1 'word12!!@#$'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry055
	self runRegex: #('[\w]+'
		' 	foo123bar	45' true (1 'foo123bar'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry056
	self runRegex: #('[\W]+'
		'fii234!!@#$34f' true (1 '!!@#$'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry057
	self runRegex: #('[^[:alnum:]]+'
		'fii234!!@#$34f' true (1 '!!@#$'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry058
	self runRegex: #('[%&[:alnum:]]+'
		'foo%3' true (1 'foo%3')
		'foo34&rt4$57a' true (1 'foo34&rt4')
		'!!@#$' false nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry059
	self runRegex: #('[[:alpha:]]+'
		' 123foo3 ' true (1 'foo')
		'123foo' true (1 'foo')
		'foo1b' true (1 'foo'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry060
	self runRegex: #('[[:cntrl:]]+'
		' a 1234asdf' false nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry061
	self runRegex: #('[[:lower:]]+'
		'UPPERlower1234' true (1 'lower')
		'lowerUPPER' true (1 'lower'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry062
	self runRegex: #('[[:upper:]]+'
		'UPPERlower1234' true (1 'UPPER')
		'lowerUPPER ' true (1 'UPPER'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry063
	self runRegex: #('[[:space:]]+'
		'2  spaces' true (1 '  '))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry064
	self runRegex: #('[^[:space:]]+'
		'  word12!!@#$  ' true (1 'word12!!@#$'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry065
	self runRegex: #('[[:graph:]]+'
		'abc' true (1 'abc'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry066
	self runRegex: #('[[:print:]]+'
		'abc' true (1 'abc'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry067
	self runRegex: #('[^[:punct:]]+'
		'!!hello,world!!' true (1 'hello'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry068
	self runRegex: #('[[:xdigit:]]+'
		'  x10FCD  ' true (1 '10FCD')
		' hgfedcba0123456789ABCDEFGH '
			true (1 'fedcba0123456789ABCDEF'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry069
	self runRegex: #('ab|cd'
		'abc' true (1 'ab')
		'abcd' true (1 'ab'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry070
	self runRegex: #('()ef'
		'def' true (1 'ef' 2 ''))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry071
	self runRegex: #('()*' nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry072
	self runRegex: #('*a' nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry073
	self runRegex: #('^*' nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry074
	self runRegex: #('$*' nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry075
	self runRegex: #('(*)b' nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry076
	self runRegex: #('$b'	'b' false nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry077
	self runRegex: #('a\' nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry078
	self runRegex: #('a\(b'
		'a(b' true (1 'a(b'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry079
	self runRegex: #('a\(*b'
		'ab' true (1 'ab')
		'a((b' true (1 'a((b'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry080
	self runRegex: #('a\\b'
		'a\b' true (1 'a\b'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry081
	self runRegex: #('abc)' nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry082
	self runRegex: #('(abc' nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry083
	self runRegex: #('((a))'
		'abc' true (1 'a' 2 'a' 3 'a'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry084
	self runRegex: #('(a)b(c)'
		'abc' true (1 'abc' 2 'a' 3 'c'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry085
	self runRegex: #('a+b+c'
		'aabbabc' true (1 'abc'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry086
	self runRegex: #('a**' nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry087
	self runRegex: #('a*?' nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry088
	self runRegex: #('(a*)*' nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry089
	self runRegex: #('(a*)+' nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry090
	self runRegex: #('(a|)*' nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry091
	self runRegex: #('(a*|b)*' nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry092
	self runRegex: #('(a+|b)*'
		'ab' true (1 'ab' 2 'b'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry093
	self runRegex: #('(a+|b)+'
		'ab' true (1 'ab' 2 'b'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry094
	self runRegex: #('(a+|b)?'
		'ab' true (1 'a' 2 'a'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry095
	self runRegex: #('[^ab]*'
		'cde' true (1 'cde'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry096
	self runRegex: #('(^)*' nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry097
	self runRegex: #('(ab|)*' nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry098
	self runRegex: #(')(' nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry099
	self runRegex: #('' 'abc' true (1 ''))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry100
	self runRegex: #('abc' '' false nil)! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry101
	self runRegex: #('a*'
		'' true '')! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry102
	self runRegex: #('abcd'
		'abcd' true (1 'abcd'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry103
	self runRegex: #('a(bc)d'
		'abcd' true (1 'abcd' 2 'bc'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry104
	self runRegex: #('([abc])*d'
		'abbbcd' true (1 'abbbcd' 2 'c'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry105
	self runRegex: #('([abc])*bcd'
		'abcd' true (1 'abcd' 2 'a'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry106
	self runRegex: #('a|b|c|d|e' 'e' true (1 'e'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry107
	self runRegex: #('(a|b|c|d|e)f'
		'ef' true (1 'ef' 2 'e'))
	"	((a*|b))*	-	c	-	-"! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry108
	self runRegex: #('abcd*efg' 
		'abcdefg' true (1 'abcdefg'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry109
	self runRegex: #('ab*' 
		'xabyabbbz' true (1 'ab')
		'xayabbbz' true (1 'a'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry110
	self runRegex: #('(ab|cd)e' 'abcde' true (1 'cde' 2 'cd'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry111
	self runRegex: #('[abhgefdc]ij' 'hij' true (1 'hij'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry112
	self runRegex: #('^(ab|cd)e' 'abcde' false nil)
	! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry113
	self runRegex: #('(abc|)def' 'abcdef' true nil)
	! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry114
	self runRegex: #('(a|b)c*d' 'abcd' true (1 'bcd' 2 'b'))
	! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry115
	self runRegex: #('(ab|ab*)bc' 'abc' true (1 'abc' 2 'a'))
	! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry116
	self runRegex: #('a([bc]*)c*' 'abc' true (1 'abc' 2 'bc'))
	! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry117
	self runRegex: #('a([bc]*)(c*d)' 'abcd' true (1 'abcd' 2 'bc' 3 'd'))
	! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry118
	self runRegex: #('a([bc]+)(c*d)' 'abcd' true (1 'abcd' 2 'bc' 3 'd'))
	! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry119
	self runRegex: #('a([bc]*)(c+d)' 'abcd' true (1 'abcd' 2 'b' 3 'cd'))
	! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry120
	self runRegex: #('a[bcd]*dcdcde' 'adcdcde' true (1 'adcdcde'))
	! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry121
	self runRegex: #('a[bcd]+dcdcde' 'adcdcde' false nil)
	! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry122
	self runRegex: #('(ab|a)b*c' 'abc' true (1 'abc'))
	! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry123
	self runRegex: #('((a)(b)c)(d)' 'abcd' true (1 'abcd' 3 'a' 4 'b' 5 'd'))
	! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry124
	self runRegex: #('[ -~]*' 'abc' true (1 'abc'))
	! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry125
	self runRegex: #('[ -~ -~]*' 'abc' true (1 'abc'))
	! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry126
	self runRegex: #('[ -~ -~ -~]*' 'abc' true (1 'abc'))
	! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry127
	self runRegex: #('[ -~ -~ -~ -~]*' 'abc' true (1 'abc'))
	! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry128
	self runRegex: #('[ -~ -~ -~ -~ -~]*' 'abc' true (1 'abc'))
	! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry129
	self runRegex: #('[ -~ -~ -~ -~ -~ -~]*' 'abc' true (1 'abc'))
	! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry130
	self runRegex: #('[ -~ -~ -~ -~ -~ -~ -~]*' 'abc' true (1 'abc'))
	! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry131
	self runRegex: #('[a-zA-Z_][a-zA-Z0-9_]*' 'alpha' true (1 'alpha'))
	! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry132
	self runRegex: #('^a(bc+|b[eh])g|.h$' 'abh' true (1 'bh' 2 nil))
	! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry133
	self runRegex: #('(bc+d$|ef*g.|h?i(j|k))' 
		'effgz' true (1 'effgz' 2 'effgz' 3 nil)
		'ij' true (1 'ij' 2 'ij' 3 'j')
		'effg' false nil
		'bcdd' false nil
		'reffgz' true (1 'effgz' 2 'effgz' 3 nil))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry134
	self runRegex: #('(((((((((a)))))))))' 'a' true (1 'a'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry135
	self runRegex: #('multiple words of text' 
		'uh-uh' false nil
		'multiple words of text, yeah' true (1 'multiple words of text'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry136
	self runRegex: #('(.*)c(.*)' 'abcde' true (1 'abcde' 2 'ab' 3 'de'))! !

!PPRegexTest methodsFor: 'testing-henry' stamp: 'lr 2/8/2010 00:45'!
testHenry137
	self runRegex: #('\((.*), (.*)\)' '(a, b)' true (2 'a' 3 'b'))! !

!PPRegexTest methodsFor: 'testing-protocol' stamp: 'lr 3/3/2010 15:50'!
testMatches
	| matcher |
	matcher := self compileRegex: '\w+$'.
	self assert: (matcher matches: 'now').
	self deny: (matcher matches: 'now is')! !

!PPRegexTest methodsFor: 'testing-protocol' stamp: 'lr 3/3/2010 15:51'!
testMatchesIn
	| matcher |
	matcher := self compileRegex: '\w+'.
	self assert: (matcher matchesIn: 'now is the time') asArray 
		= #('now' 'is' 'the' 'time')! !

!PPRegexTest methodsFor: 'testing-protocol' stamp: 'lr 3/3/2010 15:51'!
testMatchesInCollect
	| matcher |
	matcher := self compileRegex: '\w+'.
	self assert: (matcher
		matchesIn: 'now is the time'
		collect: [ :each | each reverse ]) asArray
			= #('won' 'si' 'eht' 'emit')! !

!PPRegexTest methodsFor: 'testing-protocol' stamp: 'lr 3/3/2010 15:51'!
testMatchesInDo
	| matcher expected |
	matcher := self compileRegex: '\w+'.
	expected := #('now' 'is' 'the' 'time') asOrderedCollection.
	matcher 
		matchesIn: 'now is the time'
		do: [ :each | self assert: each = expected removeFirst ].
	self assert: expected isEmpty! !

!PPRegexTest methodsFor: 'testing-protocol' stamp: 'lr 3/3/2010 15:52'!
testMatchingRangesIn
	| matcher expected |
	matcher := self compileRegex: '\w+'.
	expected := #(1 3 5 6 8 10 12 15) asOrderedCollection.
	(matcher matchingRangesIn: 'now is the time') do: [ :range |
		self assert: range first = expected removeFirst.
		self assert: range last = expected removeFirst ].
	self assert: expected isEmpty! !

!PPRegexTest methodsFor: 'testing' stamp: 'lr 2/8/2010 00:45'!
testRegex001
	self runRegex: #('^.*$' 
		'' true (1 '')
		'a' true (1 'a')
		'abc' true (1 'abc'))! !

!PPRegexTest methodsFor: 'testing' stamp: 'lr 2/8/2010 00:45'!
testRegex002
	self runRegex: #('a\w+c'
		' abb_bbc ' true (1 'abb_bbc')
		'abb-bc' false nil)! !

!PPRegexTest methodsFor: 'testing' stamp: 'lr 2/8/2010 00:45'!
testRegex003
	self runRegex: #('a\W+c'
		' abb_bbc ' false nil
		'abb-bc' false nil
		'a.,:;-&!!"#%/()={[]}+?\~*''c' true (1 'a.,:;-&!!"#%/()={[]}+?\~*''c'))! !

!PPRegexTest methodsFor: 'testing-protocol' stamp: 'lr 2/8/2010 00:45'!
testSubexpressionCount
	| matcher |
	#(('a' 1) ('a(b)' 2) ('a(b(c))' 3) ('(a)(b)' 3) ('(a(b))*' 3)) do: [ :pair |
		matcher := self matcherClass forString: pair first.
		matcher supportsSubexpressions 
			ifTrue: [ self assert: matcher subexpressionCount = pair last ] ]! !
PPRegexParser initialize!
